function tau=SRcoef(L,anm,bnm,Snmpq)
% tau = SRcoef(L,anm,bnm,Snmpq)
% Computes the translation reexpansion coefficients arranged in a
% 2-dimensional matrix
% input: L: spatial truncation number
%        anm: recurrence coefficents for thesseral coefficients
%        bnm: recurrence coefficents for sectorial coefficients
%        Snmpq: spherical multipoles used as inital values
% output: tau: reexpansion coefficients matrix
% Adapted from Nail Gumerov (2001) original code in Fortran

N2=2*L;
tau=zeros((2*L+3)^2);
% initial values tau(ll,ss,0,0)
sign=-1;
for ll=0:N2+2
    sign=-sign;
    tau(ll^2+1:(ll+1)^2,1)=sign*Snmpq((ll+1)^2:-1:ll^2+1).';
end
tau(:,1)=sqrt(4*pi)*tau(:,1);

% sectorial coefficients tau(ll,ss,mm,mm), mm=1,...,L
for mm=0:L-1
    t1=t(mm+1,mm+1);
    t2=t(mm,mm);
    t6=t(mm+1,-mm-1);
    den=bnm(t6);
    for ll=0:N2+1
        for ss=-ll:ll
            t3=t(ll,ss);
            if abs(ss-1)<=(ll+1)
                t4=t(ll+1,ss-1);
                num1=-bnm(t4)*tau(t4,t2);
            else
                num1=0;
            end
            if abs(ss-1)<=(ll-1)
                t4=t(ll-1,ss-1);
                t5=t(ll,-ss);
                num2=bnm(t5)*tau(t4,t2);
            else
                num2=0;
            end
            tau(t3,t1)=(num1+num2)/den;
        end
    end
end

% sectorial coefficients tau(ll,ss,mm,-mm), mm=1,...,L
for mm=0:L-1
    t1=t(mm,-mm);
    t7=t(mm+1,-mm-1);
    den=bnm(t7);
    for ll=0:N2+1
        for ss=-ll:ll
            t3=t(ll,ss);
            if abs(ss+1)<=(ll+1)
                t4=t(ll+1,ss+1);
                t5=t(ll+1,-ss-1);
                num1=-bnm(t5)*tau(t4,t1);
            else
                num1=0;
            end
            if abs(ss+1)<=(ll-1)
                t4=t(ll-1,ss+1);
                num2=bnm(t3)*tau(t4,t1);
            else
                num2=0;
            end
            tau(t3,t7)=(num1+num2)/den;
        end
    end
end

% symmetry relation to find tau(mm,mm,ll,ss), mm=-L,...,L
sign1=(-1)^(L-1);
for mm=-L:L
    sign1=-sign1;
    mmm=abs(mm);
    t2=t(mmm,mm);
    t3=t(mmm,-mm);
    sign2=-1;
    for ll=0:N2+1
        sign2=-sign2;
        for ss=-ll:ll
            t1=t(ll,ss);
            t4=t(ll,-ss);
            tau(t3,t4)=sign1*sign2*tau(t1,t2);
        end
    end
end

% thesseral coefficients
for ss=-L:L
    for mm=-L:L
        maxms=max(abs(ss),abs(mm));
        % propagation in ll direction
        for j=0:L-maxms-1
            ll=j+abs(ss);
            t3=t(ll+1,ss);
            t4=t(ll,ss);
            den=anm(t4);
            for nn=abs(mm)+j:N2-maxms-j
                t1=t(nn,mm);
                t2=t(nn+1,mm);
                t5=t(nn-1,mm);
                if abs(ss)<=(ll-1)
                    t6=t(ll-1,ss);
                    num2=anm(t6)*tau(t6,t1);
                else
                    num2=0;
                end
                if abs(mm)<=(nn-1)
                    num1=anm(t5)*tau(t4,t5);
                else
                    num1=0;
                end
                tau(t3,t1)=(num1-anm(t1)*tau(t4,t2)+num2)/den;
            end
        end
        % propagation in nn direction
        for j=0:L-maxms-1
            nn=j+abs(mm);
            t3=t(nn+1,mm);
            t4=t(nn,mm);
            den=anm(t4);
            for ll=abs(ss)+j:N2-maxms-j
                t1=t(ll,ss);
                t2=t(ll+1,ss);
                t5=t(ll-1,ss);
                if abs(mm)<=(nn-1)
                    t6=t(nn-1,mm);
                    num2=anm(t6)*tau(t1,t6);
                else
                    num2=0;
                end
                if abs(ss)<=(ll-1)
                    num1=anm(t5)*tau(t5,t4);
                else
                    num1=0;
                end
                tau(t1,t3)=(num1-anm(t1)*tau(t2,t4)+num2)/den;
            end
        end
        
    end
end



function lin=t(n,m)
% 2-D to 1-D mapping function
lin = (n+1)^2 - (n-m);