function c0 = coeffPsi0(a,centers,s,k,L,Nsph,c_jl)
% c0 = coeffPsi0(a,s,k,L,c_jl)
% input: a: spheres radii
%        centers: spheres centers
%        s: direction of incidence [theta_in, phi_in]
%        k: wave number
%        L: spatial truncation number
%        Nsph: number of spheres
%        c_jl: coefficients multiplying the c0(t) elements
% output: c0: (L+1)^2 x Nsph-vector of coefficients of expansion of the mean 
%             of the incident wave Psi_in(0) in spherical harmonics basis
%             on the surface of each sphere

p=L+1;      % numerical integration trucation number
w(1:p)=0;
% Gaussian nodes and weights in theta coordinate (adapted from Nail)
MYU=LegendreZeros(p);
for j=1:p
    w(j)=2*(1-MYU(j)^2)/(p*Pn(MYU(j),p-1))^2;
end;
% equispaced nodes (with equal weights) for the phi coordinate
for n=-(p-1):p-1
    Phi(n+p)=2*pi*n/(2*p-1);
end;
jl = sph_bessel(k,a,L);    % vector j_l for l=0...L. index=l+1
c0=zeros((L+1)^2,Nsph);

t=0;
for el=0:L
    % Matrix of semi normalized associated Legendre functions evaluated
    % at Gaussian nodes, with el+1 rows, one for each order m
    Ylm = sqrt((2*el+1)/(2*4*pi)) * legendre(el,MYU,'sch');
    Ylm(1,:) = sqrt(2)*Ylm(1,:);     % normalization const different for order 0
    for m=-el:el  % minus el to el
        Y = Ylm(abs(m)+1,:);         % row corresponding to m_th order

        % compute C_lm = C(t) by Gaussian integration vs conjugate of
        % spherical harmonic
 		Iext=zeros(1,Nsph);
 		for j=1:p
              Iin=zeros(1,Nsph);
              theta=acos(MYU(j));
              for n=-(p-1):p-1
                  phi=Phi(n+p);      
                  Iin = Iin + Y(j)*exp(-i*m*phi)*IncidWav(a,centers,s,k, theta,phi);
              end;
              Iin=Iin*w(j);
              Iext=Iext+Iin;
        end;
        t=t+1;                      % 2-D to linear index mapping
        c0(t,:)=Iext*2*pi/(2*p-1);  % equal weights for the inner sums (phi coordinate)
        
    end     % for m loop
    % normalizing all coefficients computed during this "el"-iteration
    for q=1:Nsph
        c0(el^2+1 : (el+1)^2,q) = c0(el^2+1 : (el+1)^2,q) * (c_jl(q,el+1) / jl(q,el+1)).';
    end
end     % for el loop

% Analytic form for coefficients c0(:)
% d=5*a; Q=1;
% jl = sqrt(pi/(2*k*d))*besselj([0+0.5:L+0.5],k*d);
% yl = sqrt(pi/(2*k*d))*bessely([0+0.5:L+0.5],k*d);
% hl = jl + i*yl;             % vector h_l(k*d) for l=0...L. index=l+2
% c0=zeros((L+1)^2,1);
% for el=0:L
%     c0(el^2+el+1)=Q*i*k* sqrt((2*el+1)/(4*pi)) * hl(el+1) * c_jl(el+1);
%     %c0(el^2+el+1)=Q*i*k* sqrt((2*el+1)/(4*pi)) * hl(el+1);
% end
