function dl = compD(k,a,sigma,L)
% dl = compD(k,a,sigma,L)
% input: k: wave number
%        a: sphere radius
%        sigma: surface admittance
%        L: required output size
% output: dl: an N-by-(L+1) vector that computes the coefficient
%           d_l= k*h'_l(k*a) + i*sigma*h_l(k*a), l=0...L, for each sphere
%           where h and h' are the Hankel function and its derivative.

for sph=1:length(a)
	jl = sqrt(pi/(2*k*a(sph)))*besselj([-1+0.5:L+0.5],k*a(sph));
	yl = sqrt(pi/(2*k*a(sph)))*bessely([-1+0.5:L+0.5],k*a(sph));
	hl = jl + i*yl;             % vector h_l for l=-1...L. index=l+2
	hlprime = hl(1:L+1) - (1/k/a(sph))*[0+1:L+1].*hl(2:L+2);     % compute derivative h'_l
	dl(sph,:) = k*hlprime + i*sigma(sph)*hl(2:L+2);             % required coefficients vector
end