function c_jl = compJ(k,a,sigma,L)
% c_jl = compJ(k,a,sigma,L)
% input: k: wave number
%        a: sphere radius
%        sigma: surface admittance
%        L: required output size
% output: c_jl: an N-by-(L+1) vector that computes the coefficient
%               k*j'_l(k*a) + i*sigma*j_l(k*a), l=0...L, for each sphere
%               where j and j' are the Bessel function and its derivative.

for sph=1:length(a)
	jl = sqrt(pi/(2*k*a(sph)))*besselj([-1+0.5:L+0.5],k*a(sph));    % vector j_l for l=-1...L. index=l+2
	jlprime = jl(1:L+1) - (1/k/a(sph))*[0+1:L+1].*jl(2:L+2);   % compute derivative j'_l
	c_jl(sph,:) = k*jlprime + i*sigma(sph)*jl(2:L+2);          % required coefficients vector
end