function lambda = compLambda(M)
% lambda = compLambda(M)
% Computes the eigenvalues of a function in the inner product of
% two vectors, corresponding to the spherical harmonic eigenvectors
% covariance function used: exp(-acos(myu)), myu=dot(x,y)
% input: M: no. of eigenvalues to compute 
%           (max order of spherical harmonic)
% output: lambda: (M+1)-vector of eigenvalues

ng=2*M+1;      % number of gauss nodes
lambda=zeros(M+1,1);
w(1:ng)=0;
% Gaussian nodes and weights (adapted from Nail)
MYU=LegendreZeros(ng);           % nodes
for j=1:ng
    w(j)=2*(1-MYU(j)^2)/(ng*Pn(MYU(j),ng-1))^2;       % weights
end;

% Compute eigenvalues
for n=0:M
    PP=legendre(n,MYU);
    P=PP(1,:);       % Legendre polynomial evaluated at Gauss nodes
    I = sum(w.* exp(-acos(MYU)) .* P);    % numerical integration
    lambda(n+1)=2*pi*I;
end