function [var,mean]=compMoments(A,L,Nw,Nsph,ngrid,k,a,ess,centers)
% [var,mean]=compMoments(A,L,Nw,Nsph,ngrid,k,a,ess,centers)
% Computes mean and variance of the discrete solution A
% input: A: matrix of coefficients of expansion of solution
%        L: spatial trubcation number
%        Nw: stochastic truncation number
%        Nsph: number of spheres
%        ngrid: no of subdivisions in the surface mesh in each coordinate
%        k: wave number
%        a: radius
%        ess: incidence direction vector
%        centers: spheres centers
% output: mean: mean of solution at each mesh point, for every sphere
%         var: variance of solution at each mesh point, for every sphere

% mesh defining the spheres' surfaces: theta*phi (outer product)
n=ngrid-1;
mean=zeros(n+1,n+1,Nsph); var=zeros(n+1,n+1,Nsph);
theta = pi*[0:n]'/n;
phi = 2*pi*[0:n]/n;
Incid=zeros(n+1,n+1,Nsph);
L2=(L+1)^2;
 d=10*a(1);          % distance of source to center of sphere

% Incident wave evaluated at surface for each sphere
for tt=1:length(theta)
    for pp=1:length(phi)
        Incid(tt,pp,:)=IncidWav(a,centers,ess,k, theta(tt),phi(pp));
    end
end

% prepare spherical harmonics' values at mesh points
Ylm=[];         % Legendre part
for el=0:L
    Ytemp = sqrt((2*el+1)/(2*4*pi)) * legendre(el,cos(theta'),'sch');
	Ytemp(1,:) = sqrt(2)*Ytemp(1,:);     % normalization const different for order 0
	Ylm = [Ylm; Ytemp];
end
clear Ytemp
expon=zeros(2*L+1,n+1);     % exponential (imaginary) part
ex=ones(1,n+1);
exx=exp(i*phi);
for m=0:L
    expon(L+1+m,:)=ex;
    expon(L+1-m,:)=conj(ex);
    ex=ex.*exx;
end
clear ex exx

% direct sum as in Elman et al. Computes mean of magnitude
for t=1:Nw
    s=1;        % index inside columns of A
    yidx=1;     % 1st row index of Ylm in current "el-block"
    sumup=zeros(n+1,n+1,Nsph);       % partial temporary sum
    for el=0:L
        for m=-el:el
            % spherical harmonic evaluated at all mesh points
            Y=Ylm(yidx+abs(m),:).'*expon(L+1+m,:);
            for q=1:Nsph        % same harmonic for each sphere
                sumup(:,:,q) = sumup(:,:,q) + A((q-1)*L2+s,t) * Y;
            end
            s=s+1;
        end
        yidx=yidx+el+1;
    end
    mean = mean + abs(sumup);    %abs(sumup);
    var = var + abs(sumup).^2;        %abs(sumup).^2;
end
mean=abs(mean)/Nw;       % abs(mean)/Nw;
% var=abs(var).^2/Nw;            % var/Nw;
var=var/Nw-mean.^2;    % central moment

% % inverted order of summation. Fast. Computes magnitude of mean. Variance suspicious
% s=1;        % index inside columns of A
% yidx=1;     % 1st row index of Ylm in current "el-block"
% for el=0:L
%     for m=-el:el
%         % spherical harmonic evaluated at all mesh points
%         Y=Ylm(yidx+abs(m),:).' * expon(L+1+m,:);
%         for q=1:Nsph        % same harmonic for each sphere
%             mean(:,:,q) = mean(:,:,q) + sum(A((q-1)*L2+s,:)) * Y;
%             % var = var + abs( sum(A(s,:).^2) ) * abs(Y);
%             %var(:,:,q) = var(:,:,q) + ...
%              %   ( sum(abs(A((q-1)*L2+s,:)).^2)/Nw - abs(sum(A((q-1)*L2+s,:))/Nw).^2 ) * abs(Y);
%         end
%         s=s+1;
%     end
%     yidx=yidx+el+1;
% end
% 
% % mean=abs(mean+Incid)*(4*pi*10*a(1))/Nw; % total wave (incident + scattered) normalized
%                                         % by incident wave at center of sphere 1
% mean=abs(mean)/Nw;       % scattered wave
% % var=abs(var)/Nw-mean.^2; % central moment
