function RHS = compRHS(k,a,L,M, c_jl,all_lambda)
% RHS = compRHS(k,a,L,M, c_jl,all_lambda)
% input: k: wave number
%        a: sphere radius
%        L: spatial truncation number
%        M: Karhunen-Loeve expansion truncation number
%        c_jl: normalizing coefficient
%        all_lambda: eigenvalues of covariance function
% output: RHS: data structure to store elements needed in computing the RHS

jl = sph_bessel(k,a,L);   % dividing coefficient
[lambda,idx] = sort(all_lambda);    % sort ascending
lambda=flipud(lambda);              % flip order (descending)
idx=flipud(idx);
RHS=zeros(2+length(a),M); % data structure used
ptr=1;          % index in sorted array lambda
r=1;            % pointer in RHS array
while r<=M
    strt=r;             % first eigenpair corresponding to this eigenvalue
    el=idx(ptr)-1;      % degree of spherical harmonic
    fin=r+2*el;         % no. of repeated eigenvalues
    if fin>M, fin=M; end
    t=el^2+1;           % combined index for order+degree of spherical harmonic
    for ir=strt:fin
        RHS(1,ir)=lambda(ptr);          % eigenvalue
        RHS(2,ir)=t;                    % order/degree of eigenfunction (harmonic)
        RHS(3:end,ir)=c_jl(:,el+1)./jl(:,el+1);  % multiplying coefficient
        t=t+1;
    end
    r=fin+1;
    ptr=ptr+1;
    if ptr>L+1
        error('Terms of KL expansion should be less than the square of spatial truncation number');
    end
end