function meanXiEta = genMeanXiEta(M,nw,alpha)
% meanXiEta = genMeanXiEta(M,nw,alpha)
% input: M: no. of terms in Karhunen-Loeve expansion
%        nw: no. of subdivisions in each stochastic coordinate direction
%        alpha: uniform distribution of random variables over interval [-alpha,alpha]
% output: meanXiEta: Nw x M matrix consisting of the elements
%                    meanXiEta(t,r) = <xi_r eta_t> t=1...Nw, r=1...M
% Adapted by Hazem El-Alfy from O. Ernst and H. Elman

hw = 2*alpha/nw;    % stochastic hypercube side length
Nw = nw^M;          % no. of stochastic elements
indx = mod( floor( ([0:Nw-1]'*ones(1,M)) ./ (ones(Nw,1)*(nw.^[0:M-1])) ), nw);
meanXiEta=zeros(Nw,M);
for r=1:M,
   for t=1:Nw,
      ji = indx(t,r);
      wl = -alpha + ji*hw;
      wr = wl + hw;
      meanXiEta(t,r) = (wr + wl) / (2*Nw);
   end
end