function plotParam3D(a,centers,ngrid,Nsph, mean,var)
% plotParam3D(a,centers,ngrid,Nsph, mean,var)
% Plots the mean and variance of the solution on the sphere surface
% input: a: sphere radius
%        centers: list of spheres centers
%        ngrid: no of subdivisions in the surface mesh in each coordinate
%        Nsph: number of spheres
%        mean: mean of solution at each mesh point
%        var: variance of solution at each mesh point
% output: the required 3D plots

n = ngrid-1;
theta = pi*(0:n)'/n;        % theta: 0 -> pi with zero on the z-axis
phi = 2*pi*(0:n)/n;         % phi: 0 -> 2pi with zero on the x-axis
% parametric sphere surface
X = sin(theta)*cos(phi);
Y = sin(theta)*sin(phi);
Z = cos(theta)*ones(size(phi));
% 3-D plots
% plot mean
figure
hold on
for q=1:Nsph
    surf(a(q)*X+centers(q,1), a(q)*Y+centers(q,2), a(q)*Z+centers(q,3), ...
        (mean(:,:,q)),'EdgeColor','interp')
end
axis equal
xlabel('x'), ylabel('y'), zlabel('z')
colorbar
title('Mean of |\psi_{scat}| at surface')
% % plotting source as a small square (will be put after variance plot)
% X=[0,0,0,0,0]; Y=[-0.1,0.1,0.1,-0.1,-0.1];
% dc=10*a(1); Z=[dc-0.1,dc-0.1,dc+0.1,dc+0.1,dc-0.1];
% patch(X,Y,Z,'y');
hold off

% plot variance
figure
hold on
for q=1:Nsph
    surf(a(q)*X+centers(q,1), a(q)*Y+centers(q,2), a(q)*Z+centers(q,3), ...
        sqrt(var(:,:,q)),'EdgeColor','interp')
end
axis equal
xlabel('x'), ylabel('y'), zlabel('z')
colorbar
title('Standard deviation of |\psi_{scat}| at surface')
hold off

% 2-D plots
% hold on
% plot(theta,mean(:,1),'-r')
% legend('deterministic')
% plot(theta,mean(:,1),'-k', theta,var(:,1),'-y')
% legend('stochastic mean', 'stochastic variance')
% set(gca,'XTick',-0:pi/4:pi)
% set(gca,'XTickLabel',{'0','pi/4','pi/2','3/4pi','pi'})
% hold off